<?php

/**
 * @file
 * Definition of Drupal\yamlform\Tests\YamlFormWizardTest.
 */

namespace Drupal\yamlform\Tests;

use Drupal\simpletest\WebTestBase;
use Drupal\yamlform\Entity\YamlForm;
use Symfony\Component\Yaml\Yaml;

/**
 * Tests for YAML form wizard.
 *
 * @group YamlForm
 */
class YamlFormWizardTest extends WebTestBase {

  use YamlFormTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['system', 'user', 'yamlform', 'yamlform_test'];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    $this->purgeSubmissions();
    parent::tearDown();
  }

  /**
   * Test YAML form wizard.
   */
  public function testWizard() {
    $yamlform = YamlForm::load('test_form_wizard_draft_preview');

    // Get initial wizard start page (Your Information).
    $this->drupalGet('yamlform/test_form_wizard_draft_preview');
    // Check progress bar is set to 'Your Information'.
    $this->assertPattern('#<li class="current">\s+<b>Your Information</b><span></span></li>#');
    // Check draft button does not exist.
    $this->assertNoFieldById('edit-draft', 'Save Draft');
    // Check next button does exist.
    $this->assertFieldById('edit-next', 'Next Page >');
    // Check first name field does exist.
    $this->assertFieldById('edit-first-name', 'John');

    // Create a login user who can save drafts.
    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    // Move to next page (Contact Information).
    $edit = [
      'first_name' => 'Jane',
    ];
    $this->drupalPostForm('yamlform/test_form_wizard_draft_preview', $edit, t('Next Page >'));
    // Check progress bar is set to 'Contact Information'.
    $this->assertPattern('#<li class="done">\s+<b>Your Information</b><span></span></li>#');
    $this->assertPattern('#<li class="current">\s+<b>Contact Information</b></li>#');
    // Check draft button does exist.
    $this->assertFieldById('edit-draft', 'Save Draft');
    // Check previous button does exist.
    $this->assertFieldById('edit-previous', '< Previous Page');
    // Check next button does exist.
    $this->assertFieldById('edit-next', 'Next Page >');
    // Check email field does exist.
    $this->assertFieldById('edit-email', 'johnsmith@example.com');
    
    // Move to previous page (Your Information) while posting data new data
    // via autosave.
    $edit = [
      'email' => 'janesmith@example.com',
    ];
    $this->drupalPostForm(NULL, $edit, t('< Previous Page'));
    // Check progress bar is set to 'Your Information'.
    $this->assertPattern('#<li class="current">\s+<b>Your Information</b><span></span></li>#');
    // Check first name set to Jane.
    $this->assertFieldById('edit-first-name', 'Jane');
    // Check gender is still set to Male.
    $this->assertFieldChecked('edit-gender-male');

    // Change gender from Male to Female.
    $edit = [
      'gender' => 'Female',
    ];
    $this->drupalPostForm(NULL, $edit, t('Save Draft'));
    // Check first name set to Jane.
    $this->assertFieldById('edit-first-name', 'Jane');
    // Check gender is now set to Female.
    $this->assertFieldChecked('edit-gender-female');

    // Move to next page (Contact Information).
    $this->drupalPostForm('yamlform/test_form_wizard_draft_preview', [], t('Next Page >'));
    // Check progress bar is set to 'Contact Information'.
    $this->assertPattern('#<li class="current">\s+<b>Contact Information</b></li>#');
    // Check email field is now janesmith@example.com.
    $this->assertFieldById('edit-email', 'janesmith@example.com');

    // Save draft which saves the 'current_page'.
    $edit = [
      'phone' => '111-111-1111',
    ];
    $this->drupalPostForm(NULL, $edit, t('Save Draft'));
    // Complete reload the form.
    $this->drupalGet('yamlform/test_form_wizard_draft_preview');
    // Check progress bar is still set to 'Contact Information'.
    $this->assertPattern('#<li class="current">\s+<b>Contact Information</b></li>#');

    // Move to last page (Your Feedback).
    $this->drupalPostForm(NULL, [], t('Next Page >'));
    // Check progress bar is set to 'Your Feedback'.
    $this->assertPattern('#<li class="current">\s+<b>Your Feedback</b></li>#');
    // Check previous button does exist.
    $this->assertFieldById('edit-previous', '< Previous Page');
    // Check next button is labeled 'Preview'.
    $this->assertFieldById('edit-next', 'Preview');
    // Check submit button does exist.
    $this->assertFieldById('edit-submit', 'Submit');

    // Move to preview.
    $edit = [
      'comments' => 'This is working fine.',
    ];
    $this->drupalPostForm(NULL, $edit, t('Preview'));
    // Check progress bar is set to 'Preview'.
    $this->assertPattern('#<li class="current">\s+<b>Preview</b></li>#');
    // Check preview values.
    $this->assertRaw('<b>Last Name</b><br/>Smith<br/><br/>');
    $this->assertRaw('<b>Gender</b><br/>Female<br/><br/>');
    $this->assertRaw('<b>Email</b><br/><a href="mailto:janesmith@example.com">janesmith@example.com</a><br/><br/>');
    $this->assertRaw('<b>Phone</b><br/><a href="tel:111-111-1111">111-111-1111</a><br/><br/>');
    $this->assertRaw('This is working fine.<br/><br/>');

    // Submit the form.
    $this->drupalPostForm(NULL, [], t('Submit'));
    // Check progress bar is set to 'Completed'.
    $this->assertPattern('#<li class="current">\s+<b>Complete</b><span></span></li>#');

    /* Custom wizard settings */
    $this->drupalLogout();

    // Check global next and previous button labels.
    \Drupal::configFactory()->getEditable('yamlform.settings')
      ->set('settings.default_wizard_next_button_label', '{global wizard next}')
      ->set('settings.default_wizard_prev_button_label', '{global wizard previous}')
      ->save();

    $this->drupalPostForm('yamlform/test_form_wizard_draft_preview', [], t('{global wizard next}'));

    // Check progress bar.
    $this->assertRaw('class="yamlform-progress-bar"');
    // Check previous button.
    $this->assertFieldById('edit-previous', '{global wizard previous}');
    // Check next button.
    $this->assertFieldById('edit-next', '{global wizard next}');

    // Check YAML form next and previous button labels.
    $yamlform->setSettings([
      'wizard_progress_bar' => FALSE,
      'wizard_next_button_label' => '{yamlform wizard next}',
      'wizard_prev_button_label' => '{yamlform wizard previous}',
      'preview_next_button_label' => '{yamlform preview next}',
      'preview_prev_button_label' => '{yamlform preview previous}',
    ]);
    $yamlform->save();

    $this->drupalPostForm('yamlform/test_form_wizard_draft_preview', [], t('{yamlform wizard next}'));

    // Check no progress bar.
    $this->assertNoRaw('class="yamlform-progress-bar"');
    // Check previous button.
    $this->assertFieldById('edit-previous', '{yamlform wizard previous}');
    // Check next button.
    $this->assertFieldById('edit-next', '{yamlform wizard next}');

    // Check custom next and previous button labels.
    $inputs = Yaml::parse($yamlform->get('inputs'));
    $inputs['contact']['#next_button_label'] = '{inputs wizard next}';
    $inputs['contact']['#prev_button_label'] = '{inputs wizard previous}';
    $yamlform->set('inputs', Yaml::dump($inputs));
    $yamlform->save();

    $this->drupalPostForm('yamlform/test_form_wizard_draft_preview', [], t('{yamlform wizard next}'));

    // Check previous button.
    $this->assertFieldById('edit-previous', '{inputs wizard previous}');
    // Check next button.
    $this->assertFieldById('edit-next', '{inputs wizard next}');
  }

}
